"""Module providing exceptions structure"""


class PeopleControlExceptions(Exception):
    """People control api base exceptions"""

    def __init__(self, message: str) -> None:
        super().__init__(message)


class BadRequestException(PeopleControlExceptions):
    """Bad Request From api"""

    def __init__(self, message: str) -> None:
        super().__init__(f"ERROR: {message}")



"""This module provide methods to work with users entity"""

from datetime import datetime

from mindsight_people_control_api.helpers.models import (
    ApiEndpoint,
    ApiPaginationResponse,
)
from mindsight_people_control_api.settings import API_ENDPOINT_USERS, DATETIME_FORMAT


class Users(ApiEndpoint):
    """This class abstract the users endpoint methods
    Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users
    """

    def __init__(self) -> None:
        super().__init__(API_ENDPOINT_USERS)

    def get_list_users(
        self,
        search: str = None,
    ) -> ApiPaginationResponse:
        """Get areas data
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/listUsers

        Args:
            search: A search term.
        """

        path = ""
        parameters = {
            "search": search,
            "page_size": self.page_size,
        }
        return ApiPaginationResponse(
            **self._base_requests.get(path=path, parameters=parameters),
            headers=self._base_requests.headers,
        )

    def get_retrieve_user(
        self,
        _id: int,
        search: str = None,
    ) -> dict:
        """Get retrieve user
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/retrieveUser

        Args:
            _id (int, Mandatory): Id of user to retrieve
            search (str, Optional): search
        """
        path = f"/{_id}"

        parameters = {
            "search": search,
        }
        return self._base_requests.get(
            path=path,
            parameters=parameters,
        )

    def post_create_user(
        self,
        username: str,
        email: str = None,
        first_name: str = None,
        last_name: str = None,
        is_superuser: bool = False,
        is_staff: bool = False,
        is_active: bool = False,
        last_login: datetime = None,
        date_joined: datetime = datetime.now(),
    ):
        """Create new area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/createUser

        Args:
            username (str, Mandatory): Username with 254 characters or fewer.
                Letters, digits and @/./+/-/_ only
            email (str, Optional): User email with 254 characters or fewer
            first_name (str, Optional): User first name with 100 characters or fewer
            last_name (str, Optional): User last name with 150 characters or fewer
            is_superuser (bool, Optional): Super user permission
            is_staff (bool, Optional): Access to admin site
            is_active (bool, Optional): Access to front site
            last_login (datetime, Optional): Datetime of last login
            date_joined (datetime, Optional): Datetime of user joined
        """
        path = ""
        data = {
            "username": username,
            "email": email,
            "first_name": first_name,
            "last_name": last_name,
            "is_superuser": is_superuser,
            "is_staff": is_staff,
            "is_active": is_active,
            "last_login": last_login.strftime(DATETIME_FORMAT) if last_login else None,
            "date_joined": date_joined.strftime(DATETIME_FORMAT)
            if date_joined
            else None,
        }

        return self._base_requests.post(path=path, json=data)

    def patch_update_user(
        self,
        _id: int,
        username: str,
        email: str = None,
        first_name: str = None,
        last_name: str = None,
        is_superuser: bool = False,
        is_staff: bool = False,
        is_active: bool = False,
        last_login: datetime = None,
        date_joined: datetime = None,
    ) -> dict:
        """Edit partial user register
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/partialUpdateUser

        Args:
            _id (int, Mandatory): Area id
            username (str, Mandatory): Username with 254 characters or fewer.
                Letters, digits and @/./+/-/_ only
            email (str, Optional): User email with 254 characters or fewer
            first_name (str, Optional): User first name with 100 characters or fewer
            last_name (str, Optional): User last name with 150 characters or fewer
            is_superuser (bool, Optional): Super user permission
            is_staff (bool, Optional): Access to admin site
            is_active (bool, Optional): Access to front site
            last_login (datetime, Optional): Datetime of last login
            date_joined (datetime, Optional): Datetime of user joined
        """
        path = f"/{_id}"
        data = {
            "username": username,
            "email": email,
            "first_name": first_name,
            "last_name": last_name,
            "is_superuser": is_superuser,
            "is_staff": is_staff,
            "is_active": is_active,
            "last_login": last_login.strftime(DATETIME_FORMAT) if last_login else None,
            "date_joined": date_joined.strftime(DATETIME_FORMAT)
            if date_joined
            else None,
        }
        return self._base_requests.patch(path=path, data=data)

    def put_full_update_user(
        self,
        _id: int,
        username: str,
        email: str = None,
        first_name: str = None,
        last_name: str = None,
        is_superuser: bool = False,
        is_staff: bool = False,
        is_active: bool = False,
        last_login: datetime = None,
        date_joined: datetime = None,
    ) -> dict:
        """Edit full user register
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/updateUser

        Args:
            _id (int, Mandatory): Area id
            username (str, Mandatory): Username with 254 characters or fewer.
                Letters, digits and @/./+/-/_ only
            email (str, Optional): User email with 254 characters or fewer
            first_name (str, Optional): User first name with 100 characters or fewer
            last_name (str, Optional): User last name with 150 characters or fewer
            is_superuser (bool, Optional): Super user permission
            is_staff (bool, Optional): Access to admin site
            is_active (bool, Optional): Access to front site
            last_login (datetime, Optional): Datetime of last login
            date_joined (datetime, Optional): Datetime of user joined
        """
        path = f"/{_id}"
        data = {
            "username": username,
            "email": email,
            "first_name": first_name,
            "last_name": last_name,
            "is_superuser": is_superuser,
            "is_staff": is_staff,
            "is_active": is_active,
            "last_login": last_login.strftime(DATETIME_FORMAT) if last_login else None,
            "date_joined": date_joined.strftime(DATETIME_FORMAT)
            if date_joined
            else None,
        }
        return self._base_requests.put(path=path, data=data)

    def delete_destroy_user(self, _id: int, search: str = None):
        """Delete user
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Users/operation/destroyUser

        Args:
            _id (int, Mandatory): User id
            search (str, Optional): Search term
        """
        path = f"/{_id}"

        parameters = {
            "search": search,
        }
        return self._base_requests.delete(path=path, parameters=parameters)



"""This module take all methods of scripts.__init__.py"""

from mindsight_people_control_api.scripts import (
    AreaRecords,
    Areas,
    EmployeeAreas,
    EmployeeManagers,
    EmployeePositions,
    EmployeeRecord,
    Employees,
    ParentAreas,
    PositionRecords,
    Positions,
    Users,
)

__doc__ = """
**mindsight_people_control_api** is a python package to providing
methods to use People Control rest api.
"""



"""This module provide methods to work with employees entity"""

from datetime import date, datetime
from typing import Literal

from mindsight_people_control_api.helpers.models import (
    ApiEndpoint,
    ApiPaginationResponse,
)
from mindsight_people_control_api.settings import (
    API_ENDPOINT_AREAS,
    API_ENDPOINT_EMPLOYEES,
    API_ENDPOINT_POSITIONS,
    DATE_FORMAT,
)
from mindsight_people_control_api.utils.aux_functions import generate_url


class Employees(ApiEndpoint):
    """This class abstract the employees endpoint methods
    Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios
    """

    def __init__(self) -> None:
        super().__init__(API_ENDPOINT_EMPLOYEES)

    def get_list_employees(
        self,
        first_name: str = None,
        last_name: str = None,
        email: str = None,
        employee_code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> ApiPaginationResponse:
        """Get areas data
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/listEmployees

        Args:
            first_name (str, Optional): Employee first name
            last_name (str, Optional): Employee last name
            email (str, Optional): Employee email
            employee_code (str, Optional): Employee code
            created__gt (str, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (str, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (str, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (str, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get areas by status
            search: A search term.
        """

        path = ""
        parameters = {
            "first_name": first_name,
            "last_name": last_name,
            "email": email,
            "employee_code": employee_code,
            "created__gt": created__gt,
            "created__lt": created__lt,
            "modified__gt": modified__gt,
            "modified__lt": modified__lt,
            "active": active,
            "search": search,
            "page_size": self.page_size,
        }
        return ApiPaginationResponse(
            **self._base_requests.get(path=path, parameters=parameters),
            headers=self._base_requests.headers,
        )

    def get_retrieve_employee(
        self,
        _id: int,
        first_name: str = None,
        last_name: str = None,
        email: str = None,
        employee_code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> dict:
        """Get retrieve employee register
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/retrieveEmployee

        Args:
            _id (int, Mandatory): Id of employee to retrieve
            first_name (str, Optional): Employee first name
            last_name (str, Optional): Employee last name
            email (str, Optional): Employee email
            employee_code (str, Optional): Employee code
            created__gt (str, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (str, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (str, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (str, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get areas by status
            search: A search term.
        """
        path = f"/{_id}"

        parameters = {
            "first_name": first_name,
            "last_name": last_name,
            "email": email,
            "employee_code": employee_code,
            "created__gt": created__gt,
            "created__lt": created__lt,
            "modified__gt": modified__gt,
            "modified__lt": modified__lt,
            "active": active,
            "search": search,
        }
        return self._base_requests.get(
            path=path,
            parameters=parameters,
        )

    def post_create_employee(
        self,
        first_name: str,
        last_name: str,
        username: str,
        email: str,
        employee_code: str,
        start_date: date,
        area: int = None,
        position: int = None,
        manager: int = None,
    ):
        """Create new employee
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/createCompleteEmployee

        Args:
            first_name (str, Mandatory): Employee first name with 200 characters or fewer
            last_name (str, Mandatory): Employee last name with 200 characters or fewer
            username (str, Mandatory): Username with 254 characters or fewer. Letters,
                digits and @/./+/-/_ only
            email (str, Mandatory): Employee email
            employee_code (str, Mandatory): The employee code
            start_date (date, Mandatory): Start date of employee
            area (int, Optional): Employee area id
            position (int, Optional): Employee position id
            manager (int, Optional): Employee manager id
        """
        path = "/create_complete"
        data = {
            "first_name": first_name,
            "last_name": last_name,
            "username": username,
            "email": email,
            "employee_code": employee_code,
            "start_date": start_date.strftime(DATE_FORMAT),
            "area": generate_url(base_path=API_ENDPOINT_AREAS, path=f"/{area}"),
            "position": generate_url(
                base_path=API_ENDPOINT_POSITIONS, path=f"/{position}"
            ),
            "manager": generate_url(
                base_path=API_ENDPOINT_EMPLOYEES, path=f"/{manager}"
            ),
        }

        return self._base_requests.post(path=path, json=data)

    def post_activate_employee(
        self,
        _id: int,
        start_date: date,
        area: int = None,
        position: int = None,
        manager: int = None,
    ):
        """Activate employee
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/activateEmployee

        Args:
            _id (int, Mandatory): Id of employee
            start_date (date, Mandatory): Start date of employee
            area (int, Optional): Employee area id
            position (int, Optional): Employee position id
            manager (int, Optional): Employee manager id
        """
        path = f"{_id}/activate"

        data = {
            "start_date": start_date.strftime(DATE_FORMAT),
            "area": generate_url(base_path=API_ENDPOINT_AREAS, path=f"/{area}"),
            "position": generate_url(
                base_path=API_ENDPOINT_POSITIONS, path=f"/{position}"
            ),
            "manager": generate_url(
                base_path=API_ENDPOINT_EMPLOYEES, path=f"/{manager}"
            ),
        }

        return self._base_requests.post(path=path, json=data)

    def post_deactivate_employee(
        self,
        _id: int,
        end_date: date,
        termination_type: Literal[
            "dismissed", "resigned", "transfer", "intern_to_full", "others"
        ] = "others",
        termination_reason: str = None,
    ):
        """Deactivate employee
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/deactivateEmployee

        Args:
            _id (int, Mandatory): Id of employee
            end_date (date, Mandatory): Start date of employee
            termination_type (str, Optional): Termination type, default "others"
            termination_reason (str, Optional): Termination reason
        """
        path = f"{_id}/deactivate"

        data = {
            "end_date": end_date.strftime(DATE_FORMAT),
            "termination_type": termination_type,
            "termination_reason": termination_reason,
        }

        return self._base_requests.post(path=path, json=data)

    def get_retrieve_current_area(self, _id: int) -> dict:
        """Retrieve employee current area record
        Args:
            _id (int, Mandatory): Employee id
        """
        path = f"/{_id}/current_area"

        return self._base_requests.get(path=path)

    def post_change_current_area(
        self, _id: int, area_id: int, start_date: date, review_access: bool = False
    ) -> dict:
        """Change employee current area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/changeCurrentAreaEmployee

        Args:
            _id (int, Mandatory): Employee id
            area_id (int, Mandatory): New employee area id
            review_access (bool, Optional): Mark if this change will have review access
        """
        path = f"/{_id}/current_area"

        payload = {
            "area": generate_url(base_path=API_ENDPOINT_AREAS, path=f"/{area_id}"),
            "start_date": start_date.strftime(DATE_FORMAT),
            "review_access": review_access,
        }

        return self._base_requests.post(path=path, json=payload)

    def get_retrieve_current_manager(self, _id: int) -> dict:
        """Retrieve employee current manager record
        Args:
            _id (int, Mandatory): Employee id
        """
        path = f"/{_id}/current_manager"

        return self._base_requests.get(path=path)

    def post_change_current_manager(
        self, _id: int, manager_id: int, start_date: date, review_access: bool = False
    ) -> dict:
        """Change employee current manager
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/changeManagerEmployee

        Args:
            _id (int, Mandatory): Employee id
            manager_id (int, Mandatory): New employee manager id
            review_access (bool, Optional): Mark if this change will have review access
        """
        path = f"/{_id}/current_manager"

        payload = {
            "manager": generate_url(
                base_path=API_ENDPOINT_AREAS, path=f"/{manager_id}"
            ),
            "start_date": start_date.strftime(DATE_FORMAT),
            "review_access": review_access,
        }

        return self._base_requests.post(path=path, json=payload)

    def get_retrieve_current_position(self, _id: int) -> dict:
        """Retrieve employee current position record
        Args:
            _id (int, Mandatory): Employee id
        """
        path = f"/{_id}/current_position"

        return self._base_requests.get(path=path)

    def post_change_current_position(
        self, _id: int, position_id: int, start_date: date, review_access: bool = False
    ) -> dict:
        """Change employee current position
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Funcionarios/operation/changePositionEmployee

        Args:
            _id (int, Mandatory): Employee id
            position_id (int, Mandatory): New employee position id
            review_access (bool, Optional): Mark if this change will have review access
        """
        path = f"/{_id}/current_position"

        payload = {
            "position": generate_url(
                base_path=API_ENDPOINT_AREAS, path=f"/{position_id}"
            ),
            "start_date": start_date.strftime(DATE_FORMAT),
            "review_access": review_access,
        }

        return self._base_requests.post(path=path, json=payload)



"""This module provide methods to work with position records entity"""
from mindsight_people_control_api.helpers.models import (
    ApiEndpoint,
    ApiPaginationResponse,
)
from mindsight_people_control_api.settings import (
    API_ENDPOINT_POSITION_RECORDS,
)


class PositionRecords(ApiEndpoint):
    """This class abstract the positions records endpoint methods
    Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Registros-de-cargo
    """

    def __init__(self) -> None:
        super().__init__(API_ENDPOINT_POSITION_RECORDS)

    def get_list_position_records(
        self,
        position: str = None,
        search: str = None,
    ) -> ApiPaginationResponse:
        """Get positions data
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Registros-de-cargo/operation/listPositionRecords

        Args:
            position (str, Optional): Positon name
            search: search
        """

        path = ""
        parameters = {
            "position": position,
            "search": search,
            "page_size": self.page_size,
        }
        return ApiPaginationResponse(
            **self._base_requests.get(path=path, parameters=parameters),
            headers=self._base_requests.headers,
        )

    def get_retrieve_positon_record(
        self,
        _id: int,
        position: str = None,
        search: str = None,
    ) -> dict:
        """Get retrieve area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Registros-de-cargo/operation/retrievePositionRecord

        Args:
            _id (int, Mandatory): A unique integer value identifying this record of position.
            position (str, Optional): Position name
            search (str, Optional): search
        """
        path = f"/{_id}"

        parameters = {
            "position": position,   
            "search": search,
        }
        return self._base_requests.get(
            path=path,
            parameters=parameters,
        )



"""This module provide methods to work with areas entity"""

from datetime import date, datetime

from mindsight_people_control_api.helpers.models import (
    ApiEndpoint,
    ApiPaginationResponse,
)
from mindsight_people_control_api.settings import (
    API_ENDPOINT_AREAS,
    DATE_FORMAT,
    DATETIME_FORMAT,
)


class Areas(ApiEndpoint):
    """This class abstract the areas endpoint methods
    Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Areas
    """

    def __init__(self) -> None:
        super().__init__(API_ENDPOINT_AREAS)

    def get_list_areas(
        self,
        name: str = None,
        code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> ApiPaginationResponse:
        """Get areas data
        Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#operation/listAreas

        Args:
            name (str, Optional): area_name
            code (str, Optional): Code of area
            created__gt (datetime, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (datetime, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (datetime, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (datetime, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get areas by status
            search: search
            }
        """

        path = ""
        parameters = {
            "name": name,
            "code": code,
            "created__gt": created__gt.strftime(DATETIME_FORMAT)
            if created__gt
            else None,
            "created__lt": created__lt.strftime(DATETIME_FORMAT)
            if created__lt
            else None,
            "modified__gt": modified__gt.strftime(DATETIME_FORMAT)
            if modified__gt
            else None,
            "modified__lt": modified__lt.strftime(DATETIME_FORMAT)
            if modified__lt
            else None,
            "active": active,
            "search": search,
            "page_size": self.page_size,
        }
        return ApiPaginationResponse(
            **self._base_requests.get(path=path, parameters=parameters),
            headers=self._base_requests.headers,
        )

    def get_retrieve_area(
        self,
        _id: int,
        name: str = None,
        code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> dict:
        """Get retrieve area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Areas/operation/retrieveArea

        Args:
            _id (int, Mandatory): Id of area to retrieve
            name (str, Optional): Name of area
            code (str, Optional): Code of area
            created__gt (datetime, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (datetime, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (datetime, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (datetime, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get areas by status
            search (str, Optional): search
        """
        path = f"/{_id}"

        parameters = {
            "name": name,
            "code": code,
            "created__gt": created__gt.strftime(DATETIME_FORMAT)
            if created__gt
            else None,
            "created__lt": created__lt.strftime(DATETIME_FORMAT)
            if created__lt
            else None,
            "modified__gt": modified__gt.strftime(DATETIME_FORMAT)
            if modified__gt
            else None,
            "modified__lt": modified__lt.strftime(DATETIME_FORMAT)
            if modified__lt
            else None,
            "active": active,
            "search": search,
        }
        return self._base_requests.get(
            path=path,
            parameters=parameters,
        )

    def post_create_area(
        self,
        code: str,
        name: str,
        start_date: date,
        parent_area: int = None,
    ):
        """Create new area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Areas/operation/createCompleteArea

        Args:
            code (str, Mandatory): Code of area
            name (str, Mandatory): Name of area
            start_date (date, Mandatory): Area start date
            parent_area (int, Optional): Parent area id
        """
        path = "/create_complete"
        data = {
            "code": code,
            "name": name,
            "start_date": start_date.strftime(DATE_FORMAT),
            "parent_area": parent_area,
        }

        return self._base_requests.post(path=path, data=data)

    def patch_edit_area(
        self,
        _id: int,
        code: str = None,
        name: str = None,
        start_date: date = None,
        end_date: date = None,
    ) -> dict:
        """Edit area and last area record
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Areas/operation/editAreaAndRecordArea

        Args:
            _id (int, Mandatory): Area id
            code (str, Optional): Code of area
            name (str, Optional): Name of area
            start_date (date, Optional): Area start date
            end_date (date, Optional): Area end date
        """
        path = f"/{_id}/edit_area_and_record"
        data = {
            "code": code,
            "name": name,
            "start_date": start_date.strftime(DATE_FORMAT) if start_date else None,
            "end_date": end_date.strftime(DATE_FORMAT) if end_date else None,
        }
        return self._base_requests.patch(path=path, data=data)

    def patch_edit_parent_area(
        self,
        _id: int,
        parent_id: int,
        start_date: date,
        end_date: date = None,
    ) -> dict:
        """Edit parent area
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Areas/operation/editParentArea

        Args:
            _id (int, Mandatory): Area id
            parent_id (int, Mandatory): id of parent area
            start_date (date, Mandatory): Parent area assignment start date
            end_date (date, Optional): Parent area assignment end date
        """
        path = f"/{_id}/edit_parent"
        data = {
            "parent_id": parent_id,
            "start_date": start_date.strftime(DATE_FORMAT) if start_date else None,
            "end_date": end_date.strftime(DATE_FORMAT) if start_date else None,
        }
        return self._base_requests.patch(path=path, data=data)



"""This module define project configurations variables"""

from decouple import config

# API Constants
API_TOKEN = config("MINDSIGHT_CP_API_TOKEN")
API_BASE_URL = config("MINDSIGHT_CP_API_URL")
API_VERSION = "v1"

# Request config
PAGE_SIZE: int = 1000
TIMEOUT: int = 600  # Default set to 600 seconds (10 minutes)

# Date formats
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
DATE_FORMAT = "%Y-%m-%d"

# Endpoints
API_ENDPOINT_EMPLOYEES = "/employees"
API_ENDPOINT_EMPLOYEE_RECORDS = "/employee_records"
API_ENDPOINT_AREAS = "/areas"
API_ENDPOINT_AREAS_RECORDS = "/area_records"
API_ENDPOINT_PARENT_AREAS = "/parent_areas"
API_ENDPOINT_EMPLOYEE_AREAS = "/employee_areas"
API_ENDPOINT_EMPLOYEE_POSITIONS = "/employee_positions"
API_ENDPOINT_EMPLOYEE_MANAGERS = "/employee_managers"
API_ENDPOINT_USERS = "/users"
API_ENDPOINT_POSITIONS = "/positions"
API_ENDPOINT_POSITION_RECORDS = "/position_records"



"""This module provide a base to use requests for api"""

from typing import Any, Literal

import requests

from mindsight_people_control_api.helpers.exceptions import BadRequestException
from mindsight_people_control_api.settings import API_TOKEN, TIMEOUT
from mindsight_people_control_api.utils.aux_functions import generate_url


class BaseRequests:
    """Aux class to communicate with mindsight api"""

    def __init__(self):
        self.__token = API_TOKEN
        self.headers = None
        self.base_path = "/"

    def __authorization_header(self) -> dict:
        return {
            "Authorization": f"Token {self.__token}",
            "Content-Type": "application/json",
        }

    @staticmethod
    def __get_not_none_data_values(data: dict):
        result = {}
        for key, value in data.items():
            if value is not None:
                result[key] = value

        return result

    def __check_response(self, response: requests.Response):
        content_text = response.text
        try:
            response.raise_for_status()

        except requests.HTTPError as http_error:

            if response.status_code == 400:
                raise BadRequestException(message=content_text) from http_error

            raise requests.HTTPError(http_error) from http_error

        except Exception as exc:
            raise exc

    def __request_helper(
        self,
        path: str,
        method: Literal["get", "post", "put", "patch", "delete"],
        headers: dict = None,
        parameters: dict = None,
        data: Any = None,
        json: Any = None,
    ):
        if not headers:
            headers = {}

        request_url = generate_url(base_path=self.base_path, path=path)
        self.headers = {**self.__authorization_header(), **headers}

        response = None
        method = method.lower()

        if method == "get":
            response = requests.get(
                url=request_url,
                headers=self.headers,
                params=parameters,
                data=data,
                timeout=TIMEOUT,
            )

        elif method == "post":
            response = requests.post(
                url=request_url,
                headers=self.headers,
                params=parameters,
                data=data,
                json=json,
                timeout=TIMEOUT,
            )

        elif method == "put":
            response = requests.put(
                url=request_url,
                headers=self.headers,
                params=parameters,
                data=data,
                json=json,
                timeout=TIMEOUT,
            )

        elif method == "patch":
            response = requests.patch(
                url=request_url,
                headers=self.headers,
                params=parameters,
                data=data,
                json=json,
                timeout=TIMEOUT,
            )

        elif method == "delete":
            response = requests.delete(
                url=request_url,
                headers=self.headers,
                params=parameters,
                data=data,
                json=json,
                timeout=TIMEOUT,
            )

        # Check response
        self.__check_response(response)

        response_json = response.json()

        return response_json

    def get(
        self,
        path: str,
        headers: dict = None,
        parameters: dict = None,
    ) -> Any:
        """Use GET method on Rest API"""
        return self.__request_helper(
            path=path, method="get", headers=headers, parameters=parameters
        )

    def post(
        self,
        path: str,
        headers: dict = None,
        parameters: dict = None,
        data: Any = None,
        json: Any = None,
    ) -> Any:
        """Use POST method on Rest API"""
        return self.__request_helper(
            path=path,
            method="post",
            headers=headers,
            parameters=parameters,
            data=data,
            json=json,
        )

    def put(
        self,
        path: str,
        headers: dict = None,
        parameters: dict = None,
        data: Any = None,
    ):
        """Use PUT method on Rest API"""
        return self.__request_helper(
            path=path,
            method="put",
            headers=headers,
            parameters=parameters,
            data=data,
        )

    def patch(
        self,
        path: str,
        headers: dict = None,
        parameters: dict = None,
        data: Any = None,
    ):
        """Use PATCH method on Rest API"""
        return self.__request_helper(
            path=path,
            method="patch",
            headers=headers,
            parameters=parameters,
            data=self.__get_not_none_data_values(data),
        )

    def delete(
        self,
        path: str,
        headers: dict = None,
        parameters: dict = None,
        data: Any = None,
    ):
        """Use DELETE method on Rest API"""
        return self.__request_helper(
            path=path,
            method="delete",
            headers=headers,
            parameters=parameters,
            data=data,
        )



"""This module import all methods scripts"""

from mindsight_people_control_api.scripts.area_records import AreaRecords
from mindsight_people_control_api.scripts.areas import Areas
from mindsight_people_control_api.scripts.employee_areas import EmployeeAreas
from mindsight_people_control_api.scripts.employee_managers import EmployeeManagers
from mindsight_people_control_api.scripts.employee_positions import EmployeePositions
from mindsight_people_control_api.scripts.employee_records import EmployeeRecord
from mindsight_people_control_api.scripts.employees import Employees
from mindsight_people_control_api.scripts.parent_areas import ParentAreas
from mindsight_people_control_api.scripts.positions import Positions
from mindsight_people_control_api.scripts.position_records import PositionRecords
from mindsight_people_control_api.scripts.users import Users



"""This module provide helpers classes to represent objects"""

import requests

from mindsight_people_control_api.helpers.base_requests import BaseRequests
from mindsight_people_control_api.settings import PAGE_SIZE, TIMEOUT


class ApiEndpoint:
    """This class represents a base api endpoint classes"""

    _base_requests: BaseRequests = BaseRequests()

    def __init__(self, base_path: str) -> None:
        self._base_requests.base_path = base_path
        self._page_size: int = PAGE_SIZE

    @property
    def page_size(self) -> int:
        """Get number of records per page."""
        return self._page_size

    @page_size.setter
    def page_size(self, value: int):

        if value <= 0:
            raise ValueError("Page size can be > 0.")

        self._page_size = value


class ApiPaginationResponse:
    """Class to work with paginated responses"""

    results: list = []

    def __init__(
        self,
        count: int,
        previous: str = None,
        results: list = None,
        headers: dict = None,
        **kwargs,
    ) -> None:
        self.count = count
        self.next = kwargs.get("next")
        self.previous = previous
        self.results.extend(results if results else [])
        self.__headers = headers

    def get_all(self):
        """Get all pages of data"""
        if self.next:
            response = requests.get(
                url=self.next,
                headers=self.__headers,
                timeout=TIMEOUT,
            )

            response.raise_for_status()
            response_data = response.json()

            self.results.extend(response_data["results"])
            self.count = response_data["count"]
            self.next = response_data["next"]
            self.previous = response_data["previous"]

            if self.next:
                self.get_all()

        return self



"""This module provide methods to work with positions entity"""

from datetime import date, datetime

from mindsight_people_control_api.helpers.models import (
    ApiEndpoint,
    ApiPaginationResponse,
)
from mindsight_people_control_api.settings import (
    API_ENDPOINT_POSITIONS,
    DATE_FORMAT,
    DATETIME_FORMAT,
)


class Positions(ApiEndpoint):
    """This class abstract the positions endpoint methods
    Reference: https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Cargos
    """

    def __init__(self) -> None:
        super().__init__(API_ENDPOINT_POSITIONS)

    def get_list_positions(
        self,
        name: str = None,
        code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> ApiPaginationResponse:
        """Get positions data
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Cargos/operation/listPositions

        Args:
            name (str, Optional): Position name
            code (str, Optional): Code of position
            created__gt (datetime, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (datetime, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (datetime, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (datetime, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get positions by status
            search: search
            }
        """

        path = ""
        parameters = {
            "name": name,
            "code": code,
            "created__gt": created__gt.strftime(DATETIME_FORMAT)
            if created__gt
            else None,
            "created__lt": created__lt.strftime(DATETIME_FORMAT)
            if created__lt
            else None,
            "modified__gt": modified__gt.strftime(DATETIME_FORMAT)
            if modified__gt
            else None,
            "modified__lt": modified__lt.strftime(DATETIME_FORMAT)
            if modified__lt
            else None,
            "active": active,
            "search": search,
            "page_size": self.page_size,
        }
        return ApiPaginationResponse(
            **self._base_requests.get(path=path, parameters=parameters),
            headers=self._base_requests.headers,
        )

    def get_retrieve_position(
        self,
        _id: int,
        name: str = None,
        code: str = None,
        created__gt: datetime = None,
        created__lt: datetime = None,
        modified__gt: datetime = None,
        modified__lt: datetime = None,
        active: str = None,
        search: str = None,
    ) -> dict:
        """Get retrieve position
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Cargos/operation/retrievePosition

        Args:
            _id (int, Mandatory): Id of position to retrieve
            name (str, Optional): Name of position
            code (str, Optional): Code of position
            created__gt (datetime, Optional): Datetime to apply filter ">=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            created__lt (datetime, Optional): Datetime to apply filter "<=" on created dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__gt (datetime, Optional): Datetime to apply filter ">=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            modified__lt (datetime, Optional): Datetime to apply filter "<=" on modified dates.
                Format "%Y-%m-%d %H:%M:%S"
            active (str, Optional): is_active: Flag to get positions by status
            search (str, Optional): search
        """
        path = f"/{_id}"

        parameters = {
            "name": name,
            "code": code,
            "created__gt": created__gt.strftime(DATETIME_FORMAT)
            if created__gt
            else None,
            "created__lt": created__lt.strftime(DATETIME_FORMAT)
            if created__lt
            else None,
            "modified__gt": modified__gt.strftime(DATETIME_FORMAT)
            if modified__gt
            else None,
            "modified__lt": modified__lt.strftime(DATETIME_FORMAT)
            if modified__lt
            else None,
            "active": active,
            "search": search,
        }
        return self._base_requests.get(
            path=path,
            parameters=parameters,
        )

    def post_create_position(
        self,
        code: str,
        name: str,
        start_date: date,
        category: str = None,
        description: str = None,
    ):
        """Create new position
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Cargos/operation/createCompletePosition

        Args:
            code (str, Mandatory): Code of position
            name (str, Mandatory): Name of position
            start_date (date, Mandatory): Position start date
            category (str, Optional): Position category
            description (str, Optional): Position description
        """
        path = "/create_complete"
        data = {
            "code": code,
            "name": name,
            "start_date": start_date.strftime(DATE_FORMAT),
            "category": category,
            "description": description,
        }

        return self._base_requests.post(path=path, json=data)

    def patch_edit_position(
        self,
        _id: int,
        code: str = None,
        name: str = None,
        category: str = None,
        description: str = None,
        start_date: date = None,
        end_date: date = None,
    ) -> dict:
        """Edit position and last position record
        Reference:
            https://controle.mindsight.com.br/stone/api/v1/docs/#tag/Cargos/operation/editPositionAndRecordPosition

        Args:
            _id (int, Mandatory): Position id
            code (str, Optional): Code of position
            name (str, Optional): Name of position
            category (str, Optional): Category of position
            description (str, Optional): Description of position
            start_date (date, Optional): Position start date
            end_date (date, Optional): Position end date
        """
        path = f"/{_id}/edit_position_and_record"
        data = {
            "code": code,
            "name": name,
            "category": category,
            "description": description,
            "start_date": start_date.strftime(DATE_FORMAT) if start_date else None,
            "end_date": end_date.strftime(DATE_FORMAT) if end_date else None,
        }
        return self._base_requests.patch(path=path, data=data)
